/*
Copyright (C) 1996-2001 Id Software, Inc.
Copyright (C) 2002-2009 John Fitzgibbons and others
Copyright (C) 2007-2008 Kristian Duske
Copyright (C) 2010-2014 QuakeSpasm developers

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "quakedef.h"
#include "libircclient.h"
#include "libirc_events.h"
#include "libirc_errors.h"
#include "libirc_options.h"

// set some default irc cvar values
cvar_t	cl_irc_channel = {"_irc_channel", "#default", CVAR_ARCHIVE}; //added by shamblernaut
cvar_t	cl_irc_nick = {"_irc_nick", "player", CVAR_ARCHIVE};

qboolean irc_connect_bool = false;
// irc_connect_bool flags IRC_Connect_f to start the connection
qboolean irc_connected_bool = false;
// should be called in IRC_Connect_f once the connection is established
char * send_msg_text;
qboolean irc_disconnect_bool = false;
// flag IRC_Connect_f to disconnect


// log file for debugging
void addlog (const char * fmt, ...)
{
	FILE * fp;
	char buf[1024];
	va_list va_alist;

	va_start (va_alist, fmt);
#if defined (_WIN32)
	_vsnprintf (buf, sizeof(buf), fmt, va_alist);
#else
	vsnprintf (buf, sizeof(buf), fmt, va_alist);
#endif
	va_end (va_alist);

	printf ("%s\n", buf);

	if ( (fp = fopen ("ircout.log", "ab")) != 0 )
	{
		fprintf (fp, "%s\n", buf);
		fclose (fp);
	}
}


void Send_IRC_Message (char * chat_buffer)
{

    if (irc_session)
	{
	send_msg_text = chat_buffer;
	irc_cmd_msg (irc_session, cl_irc_channel.string, send_msg_text);
    }
    else
    {
	Con_Printf ("Not connected\n");
    }

}


/*
==================
IRC_Password_f added by shamblernaut

Sets the IRC server password
Almost a direct copy of the name function
==================
*/

void IRC_Password_f (void)
{

		char	newPass[64];

	if (Cmd_Argc () == 1)
	{
		Con_Printf ("make sure to set this off stream\n");
		return;
	}
	if (Cmd_Argc () == 2)
		q_strlcpy(newPass, Cmd_Argv(1), sizeof(newPass));
	else
		q_strlcpy(newPass, Cmd_Args(), sizeof(newPass));
	newPass[63] = 0;

	if (cmd_source == src_command)
	{
		if (Q_strcmp(cl_irc_password.string, newPass) == 0)
			return;
		Cvar_Set ("_irc_password", newPass);

		return;
	}



}


/*
==================
IRC_Server_f added by shamblernaut

Sets the IRC servername
==================
*/

void IRC_Server_f (void)
{

	char	newServ[32];

	if (Cmd_Argc () == 1)
	{
		Con_Printf ("sets the server address\n");
		Con_Printf ("remote IRC server is: \"%s\"\n", cl_irc_server.string);
		return;
	}
	if (Cmd_Argc () == 2)
		q_strlcpy(newServ, Cmd_Argv(1), sizeof(newServ));
	else
		q_strlcpy(newServ, Cmd_Args(), sizeof(newServ));
	newServ[31] = 0;	// client_t structure actually says name[32].

	if (cmd_source == src_command)
	{
		if (Q_strcmp(cl_irc_server.string, newServ) == 0)
			return;
		Cvar_Set ("_irc_server", newServ);

		return;
	}


}



/*
==================
IRC_Port_f added by shamblernaut

Sets the remote IRC port
==================
*/

void IRC_Port_f (void)
{
	char		portNum[32];

	if (Cmd_Argc() == 1)
	{
		Con_Printf ("remote IRC port is: \"%s\"\n", cl_irc_port.string);
		return;
	}

	if (Cmd_Argc () == 2)
		q_strlcpy(portNum, Cmd_Argv(1), sizeof(portNum));
	else
		q_strlcpy(portNum, Cmd_Args(), sizeof(portNum));
	portNum[31] = 0;	// client_t structure actually says name[32].

	if (cmd_source == src_command)
	{
		if (Q_strcmp(cl_irc_port.string, portNum) == 0)
			return;
		Cvar_Set ("_irc_port", portNum);

		return;
	}


}




/*
==================
IRC_Channel_f added by shamblernaut

Sets the IRC channel name
Almost a direct copy of the name function
==================
*/

void IRC_Channel_f (void)
{

		char	newChan[32];

	if (Cmd_Argc () == 1)
	{
		Con_Printf ("Sets the IRC channel prior to connection\n");
		Con_Printf ("remote IRC channel is: \"%s\"\n", cl_irc_channel.string);
		return;
	}
	if (Cmd_Argc () == 2)
		q_strlcpy(newChan, Cmd_Argv(1), sizeof(newChan));
	else
		q_strlcpy(newChan, Cmd_Args(), sizeof(newChan));
	newChan[31] = 0;	// client_t structure actually says name[32].

	if (cmd_source == src_command)
	{
		if (Q_strcmp(cl_irc_channel.string, newChan) == 0)
			return;
		Cvar_Set ("_irc_channel", newChan);

		return;
	}



}


/*
==================
IRC_Nick_f added by shamblernaut

Sets your IRC nickname
Almost a direct copy of the name function
==================
*/

void IRC_Nick_f (void)
{

		char	newNick[32];

	if (Cmd_Argc () == 1)
	{
		Con_Printf ("Sets your nickname prior to connection\n");
		Con_Printf ("your IRC nickname is: \"%s\"\n", cl_irc_nick.string);
		return;
	}
	if (Cmd_Argc () == 2)
		q_strlcpy(newNick, Cmd_Argv(1), sizeof(newNick));
	else
		q_strlcpy(newNick, Cmd_Args(), sizeof(newNick));
	newNick[31] = 0;	// client_t structure actually says name[32].

	if (cmd_source == src_command)
	{
		if (Q_strcmp(cl_irc_nick.string, newNick) == 0)
			return;
		Cvar_Set ("_irc_nick", newNick);

		return;
	}


}


// handles all unhandled events received from the IRC server
void dump_event (irc_session_t * irc_session, const char * event, const char * origin, const char ** params, unsigned int count)
{

//	printf ("Entered function dump_event.\n"); // for debugging

	char buf[512];
	int cnt;

	buf[0] = '\0';

	for ( cnt = 0; cnt < count; cnt++ )
	{
		if ( cnt )
			strcat (buf, "|");

		strcat (buf, params[cnt]);
	}

	addlog ("Event \"%s\", origin: \"%s\", params: %d [%s]", event, origin ? origin : "NULL", cnt, buf,"\n");

}



// channel join function
void event_join (irc_session_t * irc_session, const char * event, const char * origin, const char ** params, unsigned int count)
{

    if (irc_session)
	{
	dump_event (irc_session, event, origin, params, count);
	irc_cmd_user_mode	 (irc_session, "+i");
    }
    else
    {
	Con_Printf ("Not connected\n");
    }


}

// connection event called once connected.
void event_connect (irc_session_t * irc_session, const char * event, const char * origin, const char ** params, unsigned int count)
{
	dump_event (irc_session, event, origin, params, count);
	irc_cmd_join (irc_session, cl_irc_channel.string, 0);
}

// handles messages received from server
void event_numeric (irc_session_t * irc_session, unsigned int event, const char * origin, const char ** params, unsigned int count)
{

	char buf[24];
	sprintf (buf, "%d", event);
	dump_event (irc_session, buf, origin, params, count);
}



void event_channel (irc_session_t * irc_session, const char * event, const char * origin, const char ** params, unsigned int count)
{
// original non coloured method:
//    Con_Printf ("[%s]: %s\n", origin ? origin : "someone", irc_color_strip_from_mirc(params[1]) );
// sprintf doesn't echo to the game screen, whereas Con_Printf does, so we need to use Con_Printf and 
// increment each char by 128 for coloured text, unfortunately we need to do it character by character

//	if ( !origin )
//	return;
	
	char * stripped_message;	
	stripped_message = irc_color_strip_from_mirc (params[1]);
// string containing unformatted text

	int message_len;
	message_len = (strlen(stripped_message));	
// int containing length of unformmated text string	
	
	char message_str[message_len];
// modified string with coloured text
 
	int increment_msg_len = 0;

	const char * origin_str;
	origin_str = (origin ? origin : "someone");
	
	//shiny colourful gold brackets
	char openbracket[2] = {'['};
	openbracket[0] = openbracket[0] + 53;
	char closebracket[3] = {']',' '};
	closebracket[0] = closebracket[0] + 52;
	

// make text happen while in coloured!
		for (increment_msg_len; increment_msg_len != message_len; increment_msg_len++)
		{

			if (increment_msg_len < message_len)
			{			
			
				if (increment_msg_len == 0)
				{
				
					Con_Printf ("%s", openbracket);
					Con_Printf ("%s", origin_str);
					Con_Printf ("%s", closebracket);
					message_str[increment_msg_len] = stripped_message[increment_msg_len] + 128;
					Con_Printf ("%c", message_str[0]);
			
				}
				else
				{
					message_str[increment_msg_len] = stripped_message[increment_msg_len] + 128;
					Con_Printf ("%c", message_str[increment_msg_len]);				
				}
				
			}

		}

Con_Printf ("\n");

	
}



// called from Host_InitCommands in host_cmd.c
void IRC_Con_BOOL (void)
{

	irc_connect_bool = true;

    irc_callbacks_t	callbacks;
    extern irc_session_t * irc_session;

    unsigned short port = atoi(cl_irc_port.string);

	memset (&callbacks, 0, sizeof(callbacks));

	callbacks.event_connect = event_connect;
	callbacks.event_join = event_join;
	callbacks.event_nick = dump_event;
	callbacks.event_quit = dump_event;
	callbacks.event_part = dump_event;
	callbacks.event_mode = dump_event;
	callbacks.event_topic = dump_event;
	callbacks.event_kick = dump_event;
	callbacks.event_channel = event_channel;
	callbacks.event_privmsg = dump_event;
	callbacks.event_notice = dump_event;
	callbacks.event_invite = dump_event;
	callbacks.event_umode = dump_event;
	callbacks.event_ctcp_rep = dump_event;
	callbacks.event_ctcp_action = dump_event;
	callbacks.event_unknown = dump_event;
	callbacks.event_numeric = event_numeric;
//	callbacks.event_dcc_chat_req = dump_event;
//	callbacks.event_dcc_send_req = dump_event;

   irc_session = irc_create_session( &callbacks );


    if ( !irc_session )
        {
            printf ("Could not create session\n");
            return;
        }
        else
        {

		// create the session
		irc_option_set (irc_session, LIBIRC_OPTION_DEBUG);
		irc_option_set (irc_session, LIBIRC_OPTION_STRIPNICKS);
		irc_option_set (irc_session, LIBIRC_OPTION_SSL_NO_VERIFY);


		irc_connect_bool = false;

        irc_connect (irc_session, cl_irc_server.string, port, cl_irc_password.string, cl_irc_nick.string, NULL, NULL);
		irc_connected_bool = true;

        IRC_Connect_f();

    }

}

// currently unimplemented
void IRC_Disconnect_f (void)
{
	if (irc_session)
	{
	Con_Printf ("Disconnecting...\n");
	irc_disconnect(irc_session);
    irc_connected_bool = false;
    }
    else
    {
	Con_Printf ("Not connected\n");	
	}

}

/*
================
IRC_Connect_f  Added by Shamblernaut
Main connection handler and polling code.
================
*/
void IRC_Connect_f (void)
{



			struct timeval tv;
			fd_set in_set, out_set;;
			int maxfd = 0;

			// Wait a 1000th of a sec for the events
			tv.tv_usec = 100;
			tv.tv_sec = 0;

			// remove any data from the sets' memory space
			FD_ZERO (&in_set);
			FD_ZERO (&out_set);

		if (irc_connected_bool)
		{

			// Add the IRC session descriptors - call irc_add_select_descriptors() for each active session
			if (irc_add_select_descriptors( irc_session, &in_set, &out_set, &maxfd ) )
			printf ("error in or before add_select_descriptors: %s\n", irc_strerror (irc_errno(irc_session)));


			// Call select()
			if ( select (maxfd + 1, &in_set, &out_set, 0, &tv) < 0 )
				printf ("error in select function.\n");
			// Error

			// Call irc_process_select_descriptors() for each session with the descriptor set
			if ( irc_process_select_descriptors (irc_session, &in_set, &out_set) )	// returns zero if successful
				printf ("error in or before process_select_descriptors: %s\n", irc_strerror (irc_errno(irc_session)));


		}

}



/*
================
IRC_Join_f  Added by Shamblernaut
================
*/
void IRC_Join_f (void)
{

	if (Cmd_Argc () == 1)
	{

        if (irc_session)
        {
            irc_cmd_join( irc_session, cl_irc_channel.string, 0 );
            Con_Printf ("joining channel %s.\n", cl_irc_channel.string);
            Con_Printf ("Error: %s\n", irc_strerror (irc_errno(irc_session)));
        }
        else
        {
            Con_Printf ("Not connected\n");
        }

	}
	if (Cmd_Argc () == 2)
    {

        if (irc_session)
        {
            irc_cmd_join( irc_session, Cmd_Argv(1), 0 );
            Con_Printf ("joining channel %s.\n", Cmd_Argv(1));
            Con_Printf ("Error: %s\n", irc_strerror (irc_errno(irc_session)));
        }
        else
        {
            Con_Printf ("Not connected\n");
        }

    }

	return;

}

/*
================
IRC_Part_f  Added by Shamblernaut
================
*/
void IRC_Part_f (void)
{

	if (Cmd_Argc () == 1)
	{

        if (irc_session)
        {
            irc_cmd_part( irc_session, cl_irc_channel.string);
            Con_Printf ("leaving channel %s.\n", cl_irc_channel.string);
            Con_Printf ("Error: %s\n", irc_strerror (irc_errno(irc_session)));
        }
        else
        {
            Con_Printf ("Not connected\n");
        }
	}
	if (Cmd_Argc () == 2)
    {
        if (irc_session)
        {
            irc_cmd_part( irc_session, Cmd_Argv(1));
            Con_Printf ("leaving channel %s.\n", Cmd_Argv(1));
            Con_Printf ("Error: %s\n", irc_strerror (irc_errno(irc_session)));
        }
        else
        {
            Con_Printf ("Not connected\n");
        }
    }

	return;

}


